<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2015 David Frerich (dfrerich@cdfre.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaBase\Hook;

use TYPO3\CMS\Backend\View\PageLayoutView;
use TYPO3\CMS\Backend\View\PageLayoutViewDrawItemHookInterface;

/**
 * @package     imia_base
 * @subpackage  Hook
 * @author      David Frerich <d.frerich@imia.de>
 */
class PageLayoutViewDrawItemHook implements PageLayoutViewDrawItemHookInterface
{
    /**
     * Preprocesses the preview rendering of a content element.
     *
     * @param PageLayoutView $parentObject Calling parent object
     * @param boolean $drawItem Whether to draw the item using the default functionalities
     * @param string $headerContent Header content
     * @param string $itemContent Item content
     * @param array $row Record row of tt_content
     * @return void
     */
    public function preProcess(PageLayoutView &$parentObject, &$drawItem, &$headerContent, &$itemContent, array &$row)
    {
        if ($row['header']) {
            $infoArr = array();
            $parentObject->getProcessedValue('tt_content', 'header_position,header_layout,header_link', $row, $infoArr);

            if (in_array($row['header_layout'], array(0, 100))) {
                $headerNote = '[' . $GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.xlf:labels.hidden', TRUE) . '] ';
            } else {
                $headerNote = '[H' . substr($row['header_layout'], 0, 1) . '] ';
            }

            $headerNote2 = '';
            if ($row['header_position']) {
                $headerNote2  = ' <em>(' . $row['header_position'] . ')</em>';
            }

            $headerContent =
                ($row['date']
                    ? htmlspecialchars($parentObject->itemLabels['date'] . ' ' . BackendUtility::date($row['date'])) . '<br />'
                    : '') .
                '<strong>' . $headerNote .
                    $parentObject->linkEditContent($parentObject->renderText($row['header']), $row) . $headerNote2 .
                '</strong><br />';
        }
        
        if ($row['subheader']) {
            if (in_array($row['header_layout'], array(0, 100))) {
                $headerNote = '[' . $GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.xlf:labels.hidden', TRUE) . '] ';
            } else {
                $headerNote = '[H' . ((int)substr($row['header_layout'], 0, 1) + 1) . '] ';
            }

            $itemContent .= 
                $headerNote . $parentObject->linkEditContent($parentObject->renderText($row['subheader']), $row) . 
                ($row['CType'] == 'header' ? '' : '<br style="margin-bottom: 6px" />');

            $row['subheader'] = '';
        }
	}
}