<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 David Frerich (dfrerich@cdfre.de)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace IMIA\ImiaBase\Hook;

use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Backend\Wizard\NewContentElementWizardHookInterface;

/**
 * @package     imia_base
 * @subpackage  Hook
 * @author      David Frerich <d.frerich@imia.de>
 */
class TemplavoilaNewContentElement implements NewContentElementWizardHookInterface
{
    public function manipulateWizardItems(&$wizardItems, &$parentObject)
    {
        // get tv-field
        $colpos = '';
        $colAr = GeneralUtility::trimExplode(':',$parentObject->parentRecord,1);
        if ($pos = array_search('lDEF',$colAr)) {
            if ($pos+1 < count($colAr) && $colAr[$pos+1] != 'vDef') $colpos = $colAr[$pos+1];
        }

        // remove fce
        // get remove
        $TCEFORM_TSconfig = array_merge(
            BackendUtility::getTCEFORM_TSconfig('templavoila', array('pid' => $parentObject->id)),
            BackendUtility::getTCEFORM_TSconfig('tx_fcecolumn', array('pid' => $parentObject->id))
        );

        $tceItems = array();
        if ($colpos != '' && is_array($TCEFORM_TSconfig[$colpos])) {
            foreach ($TCEFORM_TSconfig[$colpos] as $ds => $dsv) {
                // remove dot from to- or ds-type
                $dt = (substr($ds, -1) == ".") ? substr($ds, 0, -1) : $ds;

                // Marcus: get the config
                $removePattern = $dsv['removeItems'];

                // Marcus: if onlyItems included it will clear everything and will keep selected items
                if ($dsv['onlyItems']) {
                    $removePattern = '*';
                    $onlyItems = GeneralUtility::trimExplode(',',$dsv['onlyItems'],1);
                }

                // Marcus: Remove all items with asterrisk or show only selected items
                if ($removePattern == '*'){
                    foreach ($wizardItems as $item => $val) {
                        if (is_array($val['tt_content_defValues']) && isset($val['tt_content_defValues'][$dt])) {
                            if (!is_array($onlyItems) || !in_array($val['tt_content_defValues'][$dt], $onlyItems)) {
                                unset($wizardItems[$item]);
                            }
                        }
                    }
                } else {
                    $removeItems = GeneralUtility::trimExplode(',',$removePattern,1);
                    foreach ($removeItems as $ri) {
                        foreach ($wizardItems as $item => $val) {
                            if (is_array($val['tt_content_defValues']) && isset($val['tt_content_defValues'][$dt]) && $val['tt_content_defValues'][$dt]== $ri) {
                                unset($wizardItems[$item]);
                            }
                        }
                    }
                }
            }
        }

        // Get TCEFORM from TSconfig of current page
        $TCEFORM_TSconfig = BackendUtility::getTCEFORM_TSconfig('tt_content', array('pid' => $parentObject->id));
        $fceConfig = array_merge(
            $TCEFORM_TSconfig['templavoila'] ?: array(),
            $TCEFORM_TSconfig['tx_fcecolumn'] ?: array()
        );

        if ($colpos != '') {
            $TCEFORM_TSconfig['CType'] = '';

            $onlyTypesItems = $fceConfig[$colpos.'.']['onlyItems'];
            $addvals = ($onlyTypesItems) ? '*' : $fceConfig[$colpos.'.']['removeItems'];


            if (strlen($addvals) > 0) {
                $onlyTypesArray = array();
                if ($onlyTypesItems){
                    $onlyTypesArray = GeneralUtility::trimExplode(',', $onlyTypesItems,1);
                }

                if ($addvals == '*') {
                    $availableTypes = array();
                    foreach ($wizardItems as $key => $val){
                        $nextType = $val['tt_content_defValues']['CType'];
                        if ($nextType && !in_array($nextType, $availableTypes) && !in_array($nextType, $onlyTypesArray) && $nextType != 'list' && $nextType != 'templavoila_pi1') {
                            $availableTypes[] = $nextType;
                        }
                    }
                    $addvals = implode(',',$availableTypes);
                }

                $TCEFORM_TSconfig['CType']['removeItems'] .= (is_array($TCEFORM_TSconfig['CType']) && strlen($TCEFORM_TSconfig['CType']['removeItems']) > 0) ? ','.$addvals : $addvals;
            }
        }



        $removePlugins = array();
        $onlyPluginsPattern = $fceConfig[$colpos.'.']['onlyPlugins'];
        $removePluginsPattern = ($onlyPluginsPattern) ? '*' : $fceConfig[$colpos.'.']['removePlugins'];

        $onlyPlugins = GeneralUtility::trimExplode(',', $onlyPluginsPattern, 1);
        foreach ($onlyPlugins as $oplgKey => $oplgVal){
            unset($onlyPlugins[$oplgKey]);
            $onlyPlugins[] = 'plugins_'.$oplgVal;
        }

        if ($removePluginsPattern == '*'){

            foreach ($wizardItems as $key => $val){
                if (substr($key, 0, 8) == 'plugins_'){
                    $removePlugins[] = $key;
                }
            }

            //    debug($removePlugins);
        } else {
            $explodedPluginList = GeneralUtility::trimExplode(',', $removePluginsPattern,1);
            foreach ($explodedPluginList as $rmplVal){
                $removePlugins[] = 'plugins_'.$rmplVal;
            }
        }

        foreach ($removePlugins as $plgKey){
            if (!in_array($plgKey, $onlyPlugins))
                unset($wizardItems[$plgKey]);
        }

        $removeItems = array();
        if (array_key_exists('CType', $TCEFORM_TSconfig) && is_array($TCEFORM_TSconfig['CType']) && array_key_exists('removeItems', $TCEFORM_TSconfig['CType'])) {
            $removeItems = GeneralUtility::trimExplode(',',$TCEFORM_TSconfig['CType']['removeItems'],1);
        }

        $headersUsed = Array();
        // Traverse wizard items:
        foreach($wizardItems as $key => $cfg)	{
            // Exploding parameter string, if any (old style)
            if ($wizardItems[$key]['params'])	{
                // Explode GET vars recursively
                $tempGetVars = GeneralUtility::explodeUrl2Array($wizardItems[$key]['params'],TRUE);
                // If tt_content values are set, merge them into the tt_content_defValues array, unset them from $tempGetVars and re-implode $tempGetVars into the param string (in case remaining parameters are around).
                if (is_array($tempGetVars['defVals']['tt_content']))	{
                    $wizardItems[$key]['tt_content_defValues'] = array_merge(is_array($wizardItems[$key]['tt_content_defValues']) ? $wizardItems[$key]['tt_content_defValues'] : array(), $tempGetVars['defVals']['tt_content']);
                    unset($tempGetVars['defVals']['tt_content']);
                    $wizardItems[$key]['params'] = GeneralUtility::implodeArrayForUrl('',$tempGetVars);
                }
            }
            // If tt_content_defValues are defined...:

            if (is_array($wizardItems[$key]['tt_content_defValues']))	{
                // Traverse field values:
                foreach($wizardItems[$key]['tt_content_defValues'] as $fN => $fV)	{

                    if (is_array($GLOBALS['TCA']['tt_content']['columns'][$fN]))	{
                        // Get information about if the field value is OK:
                        $config = &$GLOBALS['TCA']['tt_content']['columns'][$fN]['config'];
                        $authModeDeny = $config['type']=='select' && $config['authMode'] && !$GLOBALS['BE_USER']->checkAuthMode('tt_content',$fN,$fV,$config['authMode']);
                        if ($authModeDeny || in_array($fV,$removeItems))	{
                            // Remove element all together:
                            unset($wizardItems[$key]);
                            break;
                        } else {
                            // Add the parameter:
                            $wizardItems[$key]['params'].= '&defVals[tt_content]['.$fN.']='.rawurlencode($fV);
                            $tmp = explode('_', $key);
                            $headersUsed[$tmp[0]] = $tmp[0];
                        }
                    }
                }
            }
        }

        // remove empty headlines
        foreach ($wizardItems as $key => $cfg)	{
            list ($itemCategory, $dummy) = explode('_', $key);
            if (!isset ($headersUsed[$itemCategory])) unset ($wizardItems[$key]);
        }
    }
}