<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaBase\Plugin;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * @package     imia_base
 * @subpackage  TypoScript\Parser
 * @author      David Frerich <d.frerich@imia.de>
 */
class TemplavoilaContentSlide extends \TYPO3\CMS\Frontend\Plugin\AbstractPlugin
{
    /**
     * @var array
     */
    protected $languageFallback = array();
    
    /**
     * The main method getting called as pre/postUserFunc from the 'source' property of the RECORDS TS cObject
     * rendering the Content Elements for a TV Column. Should return the tt_content entries of the first page
     * which has this value set.
     *
     * @param string
     * @param array
     * @return string
     */
    public function main($content, $conf)
    {
        if ($conf['overridePage'] || $conf['overridePage.']) {
            $overridePage = $this->cObj->stdWrap($conf['overridePage'], $conf['overridePage.']);
            $rootLine = $GLOBALS['TSFE']->sys_page->getRootLine($overridePage, $GLOBALS['TSFE']->MP);
        } else {
            $rootLine = $GLOBALS['TSFE']->rootLine;
        }

        $recordsFromTable = trim($this->cObj->stdWrap($conf['table'], $conf['table.']));
        $reverse = intval($this->cObj->stdWrap($conf['reverse'], $conf['reverse.']));
        $innerReverse = intval($this->cObj->stdWrap($conf['innerReverse'], $conf['innerReverse.']));
        $field = $this->cObj->stdWrap($conf['field'], $conf['field.']);
        $collect = intval($this->cObj->stdWrap($conf['collect'], $conf['collect.']));

        $slide = intval($this->cObj->stdWrap($conf['slide'], $conf['slide.']));
        if (!$slide) {
            $slide = -1;
        }

        $languageFallback = $this->cObj->stdWrap($conf['languageFallback'], $conf['languageFallback.']);
        if (strlen($languageFallback)) {
            $this->languageFallback = GeneralUtility::intExplode(',', $languageFallback);
        }

        // support languagevisibility extension for content slides within templavoila
        if (ExtensionManagementUtility::isLoaded('languagevisibility')) {
            $translatedLanguagesArr = $this->getAvailableLanguages(0);
            $tryLang = $GLOBALS['TSFE']->sys_language_content;
            if ( is_array( $translatedLanguagesArr ) && array_key_exists($tryLang, $translatedLanguagesArr) ) {
                if ( array_key_exists('tx_languagevisibility_fallbackorder', $translatedLanguagesArr[$tryLang] ) ) {
                    $languageFallback = $translatedLanguagesArr[ $tryLang ]['tx_languagevisibility_fallbackorder'];
                    if (strlen($languageFallback) ) {
                        $this->languageFallback = GeneralUtility::intExplode(',', $languageFallback);
                    }
                }
            }
        }

        while ($page = array_shift($rootLine)) {
            $where_hid_del_tmp = $GLOBALS['TSFE']->sys_page->where_hid_del;
            $GLOBALS['TSFE']->sys_page->where_hid_del = preg_replace('/ AND pages.doktype[^<]*<[^<]*[0-9]+/ismU', '', $GLOBALS['TSFE']->sys_page->where_hid_del);
            $page = $GLOBALS['TSFE']->sys_page->getPage($page['uid']);
            $GLOBALS['TSFE']->sys_page->where_hid_del = $where_hid_del_tmp;

            $value = $this->getPageFlexValue($page, $field);
            if ($value && $recordsFromTable) {
                $value = $this->removeHiddenRecords($value, $recordsFromTable);
            }

            if ($innerReverse) {
                $parts = GeneralUtility::trimExplode(',', $value, 1);
                $parts = array_reverse($parts);
                $value = implode(',', $parts);
            }

            if ($reverse) {
                $content = $value.(strlen($content)&&strlen($value)?',':'').$content;
            } else {
                $content .= (strlen($content)&&strlen($value)?',':'').$value;
            }

            if ($collect) {
                $collect--;
            }

            if ($slide) {
                $slide--;
            }

            if (strlen($content)&&!$collect) {
                break;
            }

            if (!$slide) {
                break;
            }
        }

        if ($conf['parseFunc.']){
            $content = $this->cObj->parseFunc($content, $conf['parseFunc.']);
        }

        return $content;
    }

    /**
     * This method removes hidden or disabled content elements from the list.
     *
     * @param string
     * @return string
     */
    protected function removeHiddenRecords($value, $recordTable)
    {
        $uids = GeneralUtility::intExplode(',', $value);
        $uidList = implode(',', $uids);
        $result = '';

        $loadDB = GeneralUtility::makeInstance('FE_loadDBGroup');
        $loadDB->start($uidList, $recordTable);
        foreach ($loadDB->tableArray as $table => $tableData) {
            if (is_array($GLOBALS['TCA'][$table])) {
                $loadDB->additionalWhere[$table] = $this->cObj->enableFields($table);
            }
        }
        $loadDB->getFromDB();

        if (is_array($loadDB->results[$recordTable])) {
            $result = array_keys($loadDB->results[$recordTable]);
            $result = implode(',', array_intersect($uids, $result));
        }

        return $result;
    }

    /**
     * This method returns the contents of the flex-field given.
     *
     * @param array
     * @param string
     * @return string
     */
    protected function getPageFlexValue($page, $field)
    {
        $xml = GeneralUtility::xml2array($page['tx_templavoila_flex']);
        $GLOBALS['TSFE']->includeTCA();

        $ds = \TYPO3\CMS\Backend\Utility\BackendUtility::getFlexFormDS(
            $GLOBALS['TCA']['pages']['columns']['tx_templavoila_flex']['config'],
            $page,
            'pages',
            'tx_templavoila_flex'
        );

        if (is_array($ds) && is_array($ds['meta'])) {
            $langChildren = intval($ds['meta']['langChildren']);
            $langDisable = intval($ds['meta']['langDisable']);
        } else {
            $langChildren = 0;
            $langDisable = 0;
        }

        $translatedLanguagesArr = $this->getAvailableLanguages($page['uid']);
        $tryLang = $GLOBALS['TSFE']->sys_language_content;

        // support languagevisibility extension for content slides within templavoila
        if (ExtensionManagementUtility::isLoaded('languagevisibility')) {
            $lvFeService = GeneralUtility::makeInstance('tx_languagevisibility_feservices');
            $languageFallback = $lvFeService::getFallbackOrderForElement($page['uid'], 'pages', $tryLang);

            if (isset($languageFallback)) {
                $this->languageFallback = GeneralUtility::intExplode(',', $languageFallback);
            }
        }

        $tryLangArr = $this->languageFallback;

        do {
            if ($langArr = $translatedLanguagesArr[$tryLang]) {
                $lKey = $langDisable ? 'lDEF' : ($langChildren ? 'lDEF' : 'l'.$langArr['ISOcode']);
                $vKey = $langDisable ? 'vDEF' : ($langChildren ? 'v'.$langArr['ISOcode']: 'vDEF');
            } else {
                $lKey = 'lDEF';
                $vKey = 'vDEF';
            }

            $value = '';
            if (is_array($xml)&&is_array($xml['data']) && is_array($xml['data']['sDEF']) && is_array($xml['data']['sDEF'][$lKey])) {
                $value = $this->getSubKey($xml['data']['sDEF'][$lKey],
                        GeneralUtility::trimExplode(',', $field, 1), $vKey);
            }
        } while ((!strlen($value))&&strlen($tryLang = array_shift($tryLangArr)));

        return $value;
    }

    /**
     * @param array $arr
     * @param array $keys
     * @param string $vKey
     * @return string
     */
    protected function getSubKey($arr, $keys, $vKey)
    {
        if (!is_array($arr)) {
            return '';
        }

        if (!count($keys)) {
            return $arr[$vKey];
        } else {
            $sKey = array_shift($keys);

            return $this->getSubKey($arr[$sKey], $keys, $vKey);
        }
    }

    /**
     * @param integer $id
     * @param boolean $onlyIsoCoded
     * @param boolean $setDefault
     * @param boolean $setMulti
     * @return array
     */
    protected function getAvailableLanguages($id = 0, $onlyIsoCoded = true, $setDefault = true, $setMulti = false)
    {
        global $LANG, $TYPO3_DB, $BE_USER, $TCA, $BACK_PATH;

        $flagAbsPath = GeneralUtility::getFileAbsFileName($TCA['sys_language']['columns']['flag']['config']['fileFolder']);
        $flagIconPath = $BACK_PATH.'../'.substr($flagAbsPath, strlen(PATH_site));

        $output = array();
        if ($id) {
            $res = $TYPO3_DB->exec_SELECTquery(
                'DISTINCT sys_language.*',
                'pages_language_overlay,sys_language',
                'pages_language_overlay.sys_language_uid = sys_language.uid AND pages_language_overlay.pid = ' . (int)$id . ' AND pages_language_overlay.deleted = 0',
                '',
                'sys_language.title'
            );
        } else {
            $res = $TYPO3_DB->exec_SELECTquery(
                'sys_language.*',
                'sys_language',
                '',
                '',
                'sys_language.title'
            );
        }

        if ($setDefault) {
            $output[0] = array(
                'uid' => 0,
                'ISOcode' => 'DEF',
            );
        }

        if ($setMulti) {
            $output[-1] = array(
                'uid' => -1,
                'ISOcode' => 'DEF',
            );
        }

        if ($res) {
            while($row = $TYPO3_DB->sql_fetch_assoc($res)) {
                \TYPO3\CMS\Backend\Utility\BackendUtility::workspaceOL('sys_language', $row);
                $output[$row['uid']]=$row;

                if ($row['static_lang_isocode']) {
                    $staticLangRow = \TYPO3\CMS\Backend\Utility\BackendUtility::getRecord('static_languages',$row['static_lang_isocode'],'lg_iso_2');
                    if ($staticLangRow['lg_iso_2']) {
                        $output[$row['uid']]['ISOcode'] = $staticLangRow['lg_iso_2'];
                    }
                }

                if (strlen ($row['flag'])) {
                    $output[$row['uid']]['flagIcon'] = @is_file($flagAbsPath.$row['flag']) ? $flagIconPath.$row['flag'] : '';
                }

                if ($onlyIsoCoded && !$output[$row['uid']]['ISOcode']) unset($output[$row['uid']]);
            }
        }

        return $output;
    }
}