<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaBase\Utility;

/**
 * @package     imia_base
 * @subpackage  Utility
 * @author      David Frerich <d.frerich@imia.de>
 */
class ExtensionUtility
{
    /**
     * @param string $extKey
     * @param string $name
     * @param string $path
     * @param string $label
     */
    static public function registerPageIcon($extKey, $name, $path, $label = null)
    {
        $iconPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extRelPath($extKey) . $path;

        $GLOBALS['TCA']['pages']['columns']['module']['config']['items'][] = array($label ?: $name, $name, $iconPath);
        \TYPO3\CMS\Backend\Sprite\SpriteManager::addTcaTypeIcon('pages', 'contains-' . $name, $iconPath);
    }

    /**
     * @param string $extKey
     * @param string $pagePath
     * @param string $fcePath
     */
    static public function registerTemplavoila($extKey, $pagePath, $fcePath)
    {
        $extPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath($extKey);
        $relExtPath = str_replace(PATH_site, '', $extPath);

        $templavoilaExtConf = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['templavoila']);
        $templavoilaExtConf['staticDS.'] = array_merge(
            is_array($templavoilaExtConf['staticDS.']) ? $templavoilaExtConf['staticDS.'] : array(),
            array(
                'enable' => '1',
                'path_fce' => $relExtPath . $fcePath,
                'path_page' => $relExtPath . $pagePath,
            )
        );
        $GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['templavoila'] = serialize($templavoilaExtConf);
    }

    /**
     * @param string $table
     * @param string|array $fields
     * @param mixed $condition
     * @param string $requestUpdateField
     */
    static public function addDisplayCond($table, $fields, $condition, $requestUpdateField = null)
    {
        if (!is_array($fields)) {
            $fields = array($fields);
        }

        foreach ($fields as $field) {
            if ($GLOBALS['TCA'][$table]['columns'][$field]['displayCond']) {
                $GLOBALS['TCA'][$table]['columns'][$field]['displayCond'] = array('AND' => array(
                    $GLOBALS['TCA'][$table]['columns'][$field]['displayCond'],
                    $condition,
                ));
            } else {
                $GLOBALS['TCA'][$table]['columns'][$field]['displayCond'] = $condition;
            }
        }

        if ($requestUpdateField) {
            $requestUpdate = array_map('trim', explode(',', $GLOBALS['TCA'][$table]['ctrl']['requestUpdate']));
            $newRequestUpdate = array($requestUpdateField);
            foreach ($requestUpdate as $requestUpdateField) {
                $newRequestUpdate[] = $requestUpdateField;
            }

            $GLOBALS['TCA'][$table]['ctrl']['requestUpdate'] = implode(',', array_unique($newRequestUpdate));
        }
    }
}