<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2014 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaBase\Utility;

use IMIA\ImiaBase\TypoScript\Parser\TypoScriptParser;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * @package     imia_base
 * @subpackage  Utility
 * @author      David Frerich <d.frerich@imia.de>
 */
class TemplateService
{
	/**
	 * Includes static template records (from static_template table, loaded through a hook) and static template files (from extensions) for the input template record row.
	 *
	 * @param string $idList A list of already processed template ids including the current; The list is on the form "[prefix]_[uid]" where [prefix] is "sys" for "sys_template" records, "static" for "static_template" records and "ext_" for static include files (from extensions). The list is used to check that the recursive inclusion of templates does not go into circles: Simply it is used to NOT include a template record/file which has already BEEN included somewhere in the recursion.
	 * @param string $templateID The id of the current template. Same syntax as $idList ids, eg. "sys_123
	 * @param array $pid The PID of the input template record
	 * @param array $row A full TypoScript template record
     * @param \TYPO3\CMS\Core\TypoScript\TemplateService $templateService
	 */
	public static function includeStaticTypoScriptSources($idList, $templateID, $pid, $row, &$templateService)
    {
		// Static Template Records (static_template): include_static is a list of static templates to include
		// Call function for link rendering:
		if (is_array($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['t3lib/class.t3lib_tstemplate.php']['includeStaticTypoScriptSources'])) {
			$_params = array(
				'idList' => &$idList,
				'templateId' => &$templateID,
				'pid' => &$pid,
				'row' => &$row
			);
			foreach ($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['t3lib/class.t3lib_tstemplate.php']['includeStaticTypoScriptSources'] as $_funcRef) {
				GeneralUtility::callUserFunction($_funcRef, $_params, $templateService);
			}
		}
		// If "Include before all static templates if root-flag is set" is set:
		if ($row['static_file_mode'] == 3 && substr($templateID, 0, 4) == 'sys_' && $row['root']) {
			$templateService->addExtensionStatics($idList, $templateID, $pid, $row);
		}
		// Static Template Files (Text files from extensions): include_static_file is a list of static files to include (from extensions)
		if (trim($row['include_static_file'])) {
			$include_static_fileArr = GeneralUtility::trimExplode(',', $row['include_static_file'], TRUE);
			// Traversing list
			foreach ($include_static_fileArr as $ISF_file) {
				if (substr($ISF_file, 0, 4) == 'EXT:') {
					list($ISF_extKey, $ISF_localPath) = explode('/', substr($ISF_file, 4), 2);
					if (strcmp($ISF_extKey, '') && ExtensionManagementUtility::isLoaded($ISF_extKey) && strcmp($ISF_localPath, '')) {
						$ISF_localPath = rtrim($ISF_localPath, '/') . '/';
						$ISF_filePath = ExtensionManagementUtility::extPath($ISF_extKey) . $ISF_localPath;
						if (@is_dir($ISF_filePath)) {
							$mExtKey = str_replace('_', '', $ISF_extKey . '/' . $ISF_localPath);
							$subrow = array(
								'constants' => @file_exists(($ISF_filePath . 'constants.txt')) ? GeneralUtility::getUrl($ISF_filePath . 'constants.txt') : '',
								'config' => @file_exists(($ISF_filePath . 'setup.txt')) ? GeneralUtility::getUrl($ISF_filePath . 'setup.txt') : '',
								'include_static' => @file_exists(($ISF_filePath . 'include_static.txt')) ? implode(',', array_unique(GeneralUtility::intExplode(',', GeneralUtility::getUrl($ISF_filePath . 'include_static.txt')))) : '',
								'include_static_file' => @file_exists(($ISF_filePath . 'include_static_file.txt')) ? implode(',', array_unique(explode(',', GeneralUtility::getUrl($ISF_filePath . 'include_static_file.txt')))) : '',
								'title' => $ISF_file,
								'uid' => $mExtKey
							);
                            if (@file_exists(($ISF_filePath . 'constants.ts'))) {
                                $subrow['constants'] = GeneralUtility::getUrl($ISF_filePath . 'constants.ts');
                            }
                            if (@file_exists(($ISF_filePath . 'setup.ts'))) {
                                $subrow['config'] = GeneralUtility::getUrl($ISF_filePath . 'setup.ts');
                            }

                            foreach (array('constants', 'config') as $key) {
                                if ($subrow[$key]) {
                                    $subrow[$key] = TypoScriptParser::checkRelativeIncludeLines($subrow[$key], 1, false, 'EXT:' . $ISF_extKey . '/' . $ISF_localPath);
                                }
                            }

							$subrow = $templateService->prependStaticExtra($subrow);
							$templateService->processTemplate($subrow, $idList . ',ext_' . $mExtKey, $pid, 'ext_' . $mExtKey, $templateID);
						}
					}
				}
			}
		}
		// If "Default (include before if root flag is set)" is set OR
		// "Always include before this template record" AND root-flag are set
		if ($row['static_file_mode'] == 1 || $row['static_file_mode'] == 0 && substr($templateID, 0, 4) == 'sys_' && $row['root']) {
			$templateService->addExtensionStatics($idList, $templateID, $pid, $row);
		}
		// Include Static Template Records after all other TypoScript has been included.
		if (is_array($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['t3lib/class.t3lib_tstemplate.php']['includeStaticTypoScriptSourcesAtEnd'])) {
			$_params = array(
				'idList' => &$idList,
				'templateId' => &$templateID,
				'pid' => &$pid,
				'row' => &$row
			);
			foreach ($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['t3lib/class.t3lib_tstemplate.php']['includeStaticTypoScriptSourcesAtEnd'] as $_funcRef) {
				GeneralUtility::callUserFunction($_funcRef, $_params, $templateService);
			}
		}
	}
}