<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2014 David Frerich (dfrerich@cdfre.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaBase\Xclass\Backend\View;

use FluidTYPO3\Flux\Utility\ExtensionNamingUtility;
use FluidTYPO3\Flux\Utility\VersionUtility;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Backend\Utility\IconUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Extbase\Object\ObjectManager;
use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;

/**
 * @package     imia_base
 * @subpackage  Xclass
 * @author      David Frerich <d.frerich@imia.de>
 */
class PageLayoutView extends \TYPO3\CMS\Backend\View\PageLayoutView
{
    /**
     * @var ObjectManager
     */
    protected static $objectManager;

    /**
     * @var \FluidTYPO3\Fluidpages\Service\ConfigurationService
     */
    protected static $fluxConfiguration;
    
    /**
     * @var array
     */
    protected static $languages = array();

    /**
     * @var array
     */
    protected static $contentConfig = array();

    /**
     * @var array
     */
    protected static $tsConfig = array();

    /**
     * @param integer $id
     * @return string
     */
    public function getTable_tt_content($id)
    {
        return self::getTable_tt_content_add($this, $id, parent::getTable_tt_content($id));
	}

    /**
     * @param array $row
     * @param bool $isRTE
     * @return string
     */
    public function tt_content_drawItem($row, $isRTE = FALSE)
    {
        $out = parent::tt_content_drawItem($row, $isRTE);
        self::tt_content_drawItem_add($this, $this->id, $row, $isRTE, $out);

        return $out;
    }

    /**
     * @param array $row
     * @return string
     */
    protected function tt_content_drawFooter(array $row) 
    {
        return self::tt_content_drawFooter_new($this, $row);
    }

    /**
     * @param array $row
     * @param integer $space
     * @param boolean $disableMoveAndNewButtons
     * @param boolean $langMode
     * @param boolean $dragDropEnabled
     * @return string
     */
    public function tt_content_drawHeader($row, $space = 0, $disableMoveAndNewButtons = FALSE, $langMode = FALSE, $dragDropEnabled = FALSE)
    {
        if (!$this->languageIconTitles) {
            $this->initializeLanguages();
        }

        return parent::tt_content_drawHeader($row, $space, $disableMoveAndNewButtons, !$this->tt_contentConfig['languageMode'], $dragDropEnabled);
    }

    /**
     * @param PageLayoutView $obj
     * @param array $row
     * @return string
     */
    public static function tt_content_drawFooter_new($obj, array $row) 
    {
        $content = '';
        $info = array();
        $obj->getProcessedValue('tt_content', 'starttime,endtime,fe_group,spaceBefore,spaceAfter', $row, $info);

        $extConf = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['imia_base']);
        if ($extConf['bootstrapVisibility']) {
            if ($row['bootstrap_visibility']) {
                $bsv = array_map('trim', explode(',', $row['bootstrap_visibility']));
                $extPath = str_replace('../typo3conf', '/typo3conf',
                    ExtensionManagementUtility::extRelPath('imia_base') . 'Resources/Public/Images/Resolution/');

                foreach ($bsv as $visibility) {
                    $label = $obj->getLanguageService()
                        ->sL('LLL:EXT:imia_base/Resources/Private/Language/DB.xlf:content_bootstrapvisibility_' . $visibility);

                    array_unshift($info, '<span><img src="' . $extPath . $visibility . '.png" alt="" style="vertical-align: middle"> ' .
                        '<span style="vertical-align: middle">' . $label . '</span></span>');
                }
            }
        }

        if ($row['section_frame']) {
            array_unshift($info, self::renderTCAField($row, 'section_frame'));
        }
        if ($row['layout']) {
            array_unshift($info, self::renderTCAField($row, 'layout'));
        }
        
        if (count($info)) {
            $content = '<div class="t3-page-ce-info">
				' . implode('<br />', $info) . '
				</div>';
        }
        
        if (!empty($content)) {
            $content = '<div class="t3-page-ce-footer">' . $content . '</div>';
        }

        return $content;
    }

    /**
     * @param array $row
     * @param $field
     * @return string
     */
    public static function renderTCAField($row, $field)
    {
        if ($row[$field]) {
            $tsConfig = self::getTsConfig($row, $field);
            $config = self::getContentConfig($row, $field);
    
            $items = array();
            foreach ($config['config']['items'] as $item) {
                $items[(int)$item[1]] = $item[0];
            }
    
            $label = $tsConfig['label'] ?: $config['label'];
            $label = $tsConfig['label.'][$GLOBALS['LANG']->lang] ?: $label;
            $label = $GLOBALS['LANG']->sL($label);
    
            $value = $row[$field];
            if (array_key_exists((int)$row[$field], $items)) {
                if ($tsConfig['altLabels.'] && array_key_exists((int)$row[$field], $tsConfig['altLabels.'])) {
                    $value = $tsConfig['altLabels.'][$row[$field]];
                } else {
                    $value = $items[$row[$field]];
                }
            } elseif ($tsConfig['addItems.'] && array_key_exists((int)$row[$field], $tsConfig['addItems.'])) {
                $value = $tsConfig['addItems.'][$row[$field]];
            }
            $value = $GLOBALS['LANG']->sL($value);
    
            return '<strong>' . $label . (strpos($label, ':') >= strlen($label) - 2 ? '' : ':') . '</strong> ' . $value;
        } else {
            return '';
        }
    }

    /**
     * @param array $row
     * @param string $field
     * @return array
     */
    public static function getContentConfig($row, $field)
    {
        if (!self::$contentConfig[$field]) {
            self::$contentConfig[$field] = $GLOBALS['TCA']['tt_content']['columns'][$field];
        }
        
        return self::$contentConfig[$field];
    }

    /**
     * @param array $row
     * @param string $field
     * @return array
     */
    public static function getTsConfig($row, $field = null)
    {
        if (!self::$tsConfig[$row['pid']]) {
            self::$tsConfig[$row['pid']] = BackendUtility::getTCEFORM_TSconfig('tt_content', $row);
        }
        
        if ($field) {
            return self::$tsConfig[$row['pid']][$field]; 
        } else {
            return self::$tsConfig[$row['pid']];
        }
    }

    /**
     * @param PageLayoutView $obj
     * @param integer $id
     * @param string $out
     */
    public static function getTable_tt_content_add($obj, $id, $out)
    {
        if (!$obj->tt_contentConfig['single'] && !$obj->tt_contentConfig['languageMode'] && !GeneralUtility::_GP('pagelayout')) {
            $languageSelector = $obj->languageSelector($id);
            $languageSelector = str_replace('<br />', '', $languageSelector);
            $languageItems = '';

            if (!$obj->tt_contentConfig['sys_language_uid']) {
                foreach (self::getLanguages($id) as $language) {
                    list($lpRecord) = BackendUtility::getRecordsByField('pages_language_overlay', 'pid', $id, 'AND sys_language_uid=' . $language['uid']);
                    BackendUtility::workspaceOL('pages_language_overlay', $lpRecord);
                    $params = '&edit[pages_language_overlay][' . $lpRecord['uid'] . ']=edit&overrideVals[pages_language_overlay][sys_language_uid]=' . $language['uid'];
                    $lPLabel = $GLOBALS['SOBE']->doc->wrapClickMenuOnIcon(
                            $obj->languageFlag($language['uid'], FALSE),
                            'pages_language_overlay',
                            $lpRecord['uid']
                        ) . ($GLOBALS['BE_USER']->check('tables_modify', 'pages_language_overlay')
                            ? '<a href="#" onclick="' . htmlspecialchars(BackendUtility::editOnClick($params, $obj->backPath))
                            . '" title="' . $GLOBALS['LANG']->getLL('edit', TRUE) . '">'
                            . IconUtility::getSpriteIcon('actions-document-open')
                            : ''
                        ) . '<span>' . htmlspecialchars(GeneralUtility::fixed_lgd_cs($lpRecord['title'], 20)) . '</span>' .
                            ($GLOBALS['BE_USER']->check('tables_modify', 'pages_language_overlay') ? '</a>' : '');

                    $languageItems .= '<p>' . $lPLabel . '</p>';
                }
            }

            $language = '';
            if ($languageSelector || $languageItems) {
                $language = '<div class="languageSelection"><div class="languageItems">' . $languageItems . '</div><div>' . $languageSelector . '</div></div>' .
                    '<style type="text/css">
                        #typo3-inner-docbody { padding-bottom: 4px !important; }
                        #typo3-inner-docbody h1 { margin-bottom: 0 !important; padding-bottom: 0 !important; border-bottom: 0 !important; }
                        #typo3-inner-docbody > div { position: relative; top: -20px; }
                    </style>';
            }
            
            // special fluidpages BackendLayout
            $out = self::getSpecialBackendLayout($obj, $id, $out);
            
            return
                $language . 
                $out . 
                self::getAdditionalStyles();
        } else {
            return $out;
        }
    }

    /**
     * @param PageLayoutView $obj
     * @param integer $id
     * @param string $out
     */
    public static function getSpecialBackendLayout($obj, $id, $out)
    {
        if (ExtensionManagementUtility::isLoaded('fluidpages')) {
            $pageOutput = '';
            $gridId = '###GRID-' . md5(uniqid(time())) . '###';

            $section = self::getFluxTemplateSection($id, 'BackendLayout', array('grid' => $gridId));
            if ($section) {
                $pageOutput = $section;
            }

            if (strpos($pageOutput, $gridId) !== false) {
                $pageOutput = str_replace($gridId, $out, $pageOutput);
            } else {
                $pageOutput .= $out;
            }
        } else {
            $pageOutput = $out;
        }
        
        return $pageOutput;
    }

    /**
     * @param integer $id
     * @param string $sectionName
     * @param array $additionalVariables
     * @return null|string
     */
    public static function getFluxTemplateSection($id, $sectionName, $additionalVariables) 
    {
        $record = self::getObjectManager()
            ->get('FluidTYPO3\\Flux\\Service\\WorkspacesAwareRecordService')
            ->getSingle('pages', '*', $id);

        $provider = self::getFluxProvider($record);

        $out = null;
        if ($provider) {
            $templatePathAndFilename = $provider->getTemplatePathAndFilename($record);
            
            if (file_exists($templatePathAndFilename)) {
                $extensionKey = $provider->getExtensionKey($record);
                $extensionName = ExtensionNamingUtility::getExtensionName($extensionKey);
                $paths = $provider->getTemplatePaths($record);
                
                $variables = self::getFluxConfigurationService()
                    ->convertFlexFormContentToArray($record[$provider->getFieldName($record)]);
                $variables['record'] = $record;
                $variables = array_merge($variables, $additionalVariables);

                if (VersionUtility::assertExtensionVersionIsAtLeastVersion('fluidpages', 3, 2)) {
                    $viewContext = new \FluidTYPO3\Flux\View\ViewContext($templatePathAndFilename, $extensionKey);
                    $viewContext->setVariables($variables);
                    $viewContext->setTemplatePaths(new \FluidTYPO3\Flux\View\TemplatePaths($paths));

                    $exposedView = self::getFluxConfigurationService()
                        ->getPreparedExposedTemplateView($viewContext);
                } else {
                    $exposedView = self::getFluxConfigurationService()
                        ->getPreparedExposedTemplateView($extensionName, 'Flux', $paths, $variables);
                }

                $exposedView->setTemplatePathAndFilename($templatePathAndFilename);

                $configurationManager = self::getObjectManager()
                    ->get('TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface');
                
                $existingContentObject = $configurationManager->getContentObject();
                $contentObject = new ContentObjectRenderer();
                $contentObject->start($record, 'pages');
                $configurationManager->setContentObject($contentObject);
                
                $previewContent = $exposedView->renderStandaloneSection($sectionName, $variables);
                
                $configurationManager->setContentObject($existingContentObject);
                
                if ($previewContent) {
                    $out = $previewContent; 
                }
            }
        }
        
        return $out;
    }

    /**
     * @param array $record
     * @return \FluidTYPO3\Flux\Provider\ProviderInterface
     */
    public static function getFluxProvider($record)
    {
        $provider = null;

        try {
            if ($record) {
                $provider = self::getFluxConfigurationService()
                    ->resolvePrimaryConfigurationProvider('pages', 'tx_fed_page_flexform', $record);

                $action = $provider->getControllerActionFromRecord($record);
                if (empty($action)) {
                    $provider = null;
                }
            }
        } catch (\Exception $e) {}
        
        return $provider;
    }

    /**
     * @return ConfigurationService
     */
    public static function getFluxConfigurationService()
    {
        if (!self::$fluxConfiguration) {
            self::$fluxConfiguration = self::getObjectManager()
                ->get('FluidTYPO3\Fluidpages\Service\ConfigurationService');
        }
        
        return self::$fluxConfiguration;
    }

    /**
     * @return ObjectManager
     */
    public static function getObjectManager()
    {
        if (!self::$objectManager) {
            self::$objectManager = GeneralUtility::makeInstance('TYPO3\CMS\Extbase\Object\ObjectManager');
        }
        
        return self::$objectManager;
    }

    /**
     * @param PageLayoutView $this
     * @param integer $id
     * @param array $row
     * @param boolean $isRTE
     * @param string $out
     */
    public static function tt_content_drawItem_add($obj, $id, $row, $isRTE, &$out)
    {
        if (!$obj->tt_contentConfig['sys_language_uid'] && $row['sys_language_uid'] == 0 && !$obj->tt_contentConfig['languageMode'] && !GeneralUtility::_GP('pagelayout')) {
            $langElements = '';
            $langRows = array();
            foreach ($GLOBALS['TYPO3_DB']->exec_SELECTgetRows('*', 'tt_content', 'deleted = 0 AND l18n_parent = ' . $row['uid']) as $langRow) {
                $langRows[$langRow['sys_language_uid']] = $langRow;
            };

            foreach (self::getLanguages($id) as $language) {
                if (array_key_exists($language['uid'], $langRows)) {
                    $langRow = $langRows[$language['uid']];
                    $obj->tt_contentData['nextThree'][$langRow['uid']] = $langRow['uid'];
                    $langElements .= $obj->tt_content_drawHeader($langRow, 0, true, true, false) . '</div>';
                } else {
                    $langElements .= '
                        <h4 class="t3-page-ce-header create-new-language">
                            <div class="t3-row-header" id="ext-gen25">
                                <span class="ce-icons-left">
                                    ' . $obj->languageFlag($language['uid'], false) . '
                                </span>
                                <span class="ce-icons">' .
                                    $obj->newLanguageButton(array($row['uid']), $language['uid']) . '
                                </span>
                            </div>
                        </h4>
                        <div class="t3-page-ce-body"></div>
                    ';
                }
            }

            $out .= '<div class="language-contents">' . $langElements . '</div>';
        }
    }

    /**
     * @param integer $id
     * @return array
     */
    public static function getLanguages($id)
    {
        if (!array_key_exists($id, self::$languages)) {
            self::$languages[$id] = array();
            $res = $GLOBALS['SOBE']->exec_languageQuery($id);
            while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)) {
                if ($GLOBALS['BE_USER']->checkLanguageAccess($row['uid'])) {
                    self::$languages[$id][] = $row;
                }
            }
        }

        return self::$languages[$id];
    }

    /**
     * @return string
     */
    public static function getAdditionalStyles()
    {
        $styles = '.languageSelection { border-bottom: 1px solid #cdcdcd; position: relative; top: -15px; text-align: right; min-height: 40px; padding-bottom: 10px }';
        $styles .= '.languageSelection .languageItems { display: inline-block; text-align: left; }';
        $styles .= '.languageSelection .languageItems p { margin: 6px 0; }';
        $styles .= '.languageSelection .languageItems p * { vertical-align: middle }';

        $styles .= '.language-contents { margin: 10px -5px -5px; }';
        $styles .= '.language-contents h4 { padding: 0 5px; }';
        $styles .= '.language-contents .t3-row-header { background: #f1f1f1 !important; }';
        $styles .= '.language-contents .t3-page-ce-body { border-color: #cdcdcd !important; }';
        $styles .= '.language-contents .t3-page-ce-body:last-child { display: none; }';
        $styles .= '.language-contents .create-new-language .ce-icons a { font-size: 11px; color: #444; font-weight: normal; font-style: italic }';
        $styles .= '.language-contents .create-new-language .t3-icon { visibility: visible !important }';
        $styles .= '.language-contents .create-new-language .t3-row-header { opacity: 0.8 }';
        $styles .= '.language-contents .t3-page-lang-copyce { margin: 0 !important; padding: 0 !important; font-size: 10px }';
        $styles .= '.language-contents .t3-page-lang-copyce input { margin: 0 !important }';
        //$styles .= '.language-contents .ce-icons-left > a { display: none !important }';

        // optimizing the basic styles
        $styles .= '.t3-gridTable { margin: 0 !important; }';
        $styles .= '.t3-gridContainer { margin: 0 -12px !important; }';
        $styles .= '#typo3-inner-docbody h1 { margin-bottom: 15px; padding-bottom: 15px; border-bottom: 1px solid #cdcdcd; }';
        $styles .= '#typo3-inner-docbody .t3-page-ce .t3-row-header .ce-icons, #typo3-inner-docbody .t3-page-ce .t3-row-header .ce-icons-left { visibility: visible; }';
        $styles .= '.t3-page-ce .t3-page-ce .t3-page-ce-body { border-color: #cacaca; }';
        $styles .= '.t3-page-ce .t3-page-ce .t3-page-ce-header .t3-row-header { background-color: #cacaca; }';
        $styles .= '#typo3-inner-docbody .t3-page-ce:hover > * > .t3-page-ce-body { border-color: #5b5b5b; }';
        $styles .= '#typo3-inner-docbody .t3-page-ce:hover > * > .t3-page-ce-header .t3-row-header { background-color: #5b5b5b; }';
        $styles .= '#typo3-inner-docbody h1 + .typo3-message { top: 20px !important }';

        return '<style type="text/css">' . $styles . '</style>';
    }
}