<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaBase\XClass\Templavoila\Module;

/**
 * @package     imia_base
 * @subpackage  Xclass
 * @author      David Frerich <d.frerich@imia.de>
 */
class Module1 extends \tx_templavoila_module1
{
	/**
	 * Rendering the preview of content for Page module.
	 *
	 * @param	array
	 * @param	array
	 * @param	array
	 * @param	string
	 * @param	string
	 * @return	string
	 */
	public function render_previewData($previewData, $elData, $ds_meta, $languageKey, $sheet)
    {
		$this->currentElementBelongsToCurrentPage = $elData['table'] == 'pages' || $elData['pid'] == $this->rootElementUid_pidForContent;

        // General preview of the row:
		$previewContent = is_array($previewData['fullRow']) && $elData['table']=='tt_content' ?
            $this->render_previewContent($previewData['fullRow']) : '';

        // Preview of FlexForm content if any:
		if (is_array($previewData['sheets'][$sheet])) {
            // Define l/v keys for current language:
			$langChildren = intval($ds_meta['langChildren']);
			$langDisable = intval($ds_meta['langDisable']);
			$lKey = $langDisable ? 'lDEF' : ($langChildren ? 'lDEF' : 'l'.$languageKey);
			$vKey = $langDisable ? 'vDEF' : ($langChildren ? 'v'.$languageKey : 'vDEF');

			foreach($previewData['sheets'][$sheet] as $fieldData)	{
				if (!isset($fieldData['tx_templavoila']['preview']) || $fieldData['tx_templavoila']['preview'] != 'disable') {
                    $TCEformsConfiguration = $fieldData['TCEforms']['config'];
                    $TCEformsLabel = $this->localizedFFLabel($fieldData['TCEforms']['label'], 1);	// title for non-section elements

                    if ($fieldData['type']=='array')	{	// Making preview for array/section parts of a FlexForm structure:;
                        if(is_array($fieldData['childElements'][$lKey])) {
                            $subData = $this->render_previewSubData($fieldData['childElements'][$lKey], $elData['table'], $previewData['fullRow']['uid'], $vKey);
                            $previewContent .= $this->link_edit($subData, $elData['table'], $previewData['fullRow']['uid']);
                        } else {
                            // no child elements found here
                        }
                    } else {
                        // Preview of flexform fields on top-level:
                        $fieldValue = $fieldData['data'][$lKey][$vKey];

                        if ($TCEformsConfiguration['type'] == 'group') {
                            if ($TCEformsConfiguration['internal_type'] == 'file')	{
                                // Render preview for images:
                                $thumbnail = \TYPO3\CMS\Backend\Utility\BackendUtility::thumbCode(
                                    array('dummyFieldName'=> $fieldValue),
                                    '',
                                    'dummyFieldName',
                                    $this->doc->backPath,
                                    '',
                                    $TCEformsConfiguration['uploadfolder']
                                );

                                $previewContent .= '<p><strong>' . $TCEformsLabel . ':</strong> ' . $thumbnail . '</p>';
                            } elseif ($TCEformsConfiguration['internal_type'] === 'db') {
                                if (!$this->renderPreviewDataObjects) {
                                    $this->renderPreviewDataObjects = $this->hooks_prepareObjectsArray ('renderPreviewDataClass');
                                }
                                if (isset($this->renderPreviewDataObjects[$TCEformsConfiguration['allowed']])
                                    && method_exists($this->renderPreviewDataObjects[$TCEformsConfiguration['allowed']], 'render_previewData_typeDb')) {
                                    $previewContent .= $this->renderPreviewDataObjects[$TCEformsConfiguration['allowed']]->render_previewData_typeDb ($fieldValue, $fieldData, $previewData['fullRow']['uid'], $elData['table'], $this);
                                }
                            }
                        } elseif ($TCEformsConfiguration['type'] == 'inline' && $TCEformsConfiguration['foreign_table'] == 'sys_file_reference'
                          && $TCEformsConfiguration['foreign_match_fields']['fieldname']) {
                            // Render preview for FAL images:
                            $table = $elData['table'];
                            $fieldName = $TCEformsConfiguration['foreign_match_fields']['fieldname'];

                            $tempTCAConfig = $GLOBALS['TCA'][$table]['columns'][$fieldName]['config'];

                            // simulate TCA for FAL-field
                            $GLOBALS['TCA'][$table]['columns'][$fieldName]['config'] = $TCEformsConfiguration;
                            $thumbnail = \TYPO3\CMS\Backend\Utility\BackendUtility::thumbCode(
                                array('uid' => $elData['uid']),
                                $table,
                                $fieldName,
                                $this->doc->backPath
                            );
                            $GLOBALS['TCA'][$table]['columns'][$fieldName]['config'] = $tempTCAConfig;

                            $previewContent .= '<p><strong>' . $TCEformsLabel . ':</strong> ' . $thumbnail . '</p>';
                        } elseif (($TCEformsConfiguration['type'] == 'select' && $TCEformsConfiguration['foreign_table'])
                          || ($TCEformsConfiguration['type'] == 'inline' && $TCEformsConfiguration['foreign_table'])
                          || ($TCEformsConfiguration['type'] == 'group' && $TCEformsConfiguration['internal_type'] == 'db' && $TCEformsConfiguration['allowed'])) {
                            // Render preview for FAL images:
                            $table = $elData['table'];

                            $formEngine = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\CMS\Backend\Form\FormEngine');
                            $formEngine->enableClickMenu = true;

                            $perms_clause = $GLOBALS['BE_USER']->getPagePermsClause(1);

                            if ($TCEformsConfiguration['type'] == 'group') {
                                $fieldTables = array_map('trim', explode(',', $TCEformsConfiguration['allowed']));
                            } else {
                                $fieldTables = array($TCEformsConfiguration['foreign_table']);
                            }

                            foreach ($fieldTables as $this_table) {
                                foreach (array_map('intval', explode(',', $fieldValue)) as $this_uid) {
                                    if (count($fieldTables) == 1 || strpos($this_uid, $this_table) !== false) {
                                        $this_uid = (int)str_replace($this_table, '', $this_uid);
                                        $rr = \TYPO3\CMS\Backend\Utility\BackendUtility::getRecordWSOL($this_table, $this_uid);
                                        
                                        if ($rr) {
                                            $thumbnails .= '<span class="nobr">' . $formEngine->getClickMenu(
                                                    \TYPO3\CMS\Backend\Utility\IconUtility::getSpriteIconForRecord($this_table, $rr, array(
                                                'style' => 'vertical-align:top',
                                                'title' => htmlspecialchars((\TYPO3\CMS\Backend\Utility\BackendUtility::getRecordPath(
                                                    $rr['pid'], $perms_clause, 15) . ' [UID: ' . $rr['uid'] . ']'))
                                            )), $this_table, $this_uid) . '&nbsp;' . $this->link_edit(\TYPO3\CMS\Backend\Utility\BackendUtility::getRecordTitle
                                                ($this_table, $rr, TRUE) . ' <span class="typo3-dimmed"><em>[' . $rr['uid'] . ']</em></span>',
                                                $elData['table'], $previewData['fullRow']['uid']) . '</span><br>';
                                        }
                                    }
                                }
                            }

                            $previewContent .= '<p><strong>' . $TCEformsLabel . ':</strong>' . ($thumbnails ? '<br>' . $thumbnails : '') . '</p>';
                        } elseif ($TCEformsConfiguration['type'] != '') {
                            // Render for everything else:
                            $previewContent .= '<p><strong>' . $TCEformsLabel . ':</strong> ' . (!$fieldValue ? '' :
                                $this->link_edit(htmlspecialchars(\TYPO3\CMS\Core\Utility\GeneralUtility::fixed_lgd_cs(
                                strip_tags($fieldValue),200)), $elData['table'], $previewData['fullRow']['uid'])) . '</p>';
                        }
                    }
                }
			}
		}

		return $previewContent;
	}
}